package com.trafficparrot.examples.aws;

import com.amazon.sqs.javamessaging.ProviderConfiguration;
import com.amazon.sqs.javamessaging.SQSConnection;
import com.amazon.sqs.javamessaging.SQSConnectionFactory;
import com.amazonaws.services.sqs.AmazonSQS;
import org.junit.Rule;
import org.junit.Test;

import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import java.io.IOException;

import static com.trafficparrot.examples.aws.TrafficParrotAmazonSQS.trafficParrotAmazonSQS;

public class AmazonSQSIntegrationTest {

    private static final int VIRTUAL_SERVICE_PORT = 18081;
    private static final String VIRTUAL_SERVICE_URL = "http://localhost:" + VIRTUAL_SERVICE_PORT + "/aws/sqs";

    @Rule
    public TrafficParrotAmazonSQS trafficParrotAmazonSQS = trafficParrotAmazonSQS(VIRTUAL_SERVICE_PORT);

    @Test
    public void sendMessagesToQueuesAndVerifyTheyArrived() throws JMSException, IOException {
        AmazonSQS amazonSQS = AmazonSQSClient.amazonSQSClient(VIRTUAL_SERVICE_URL);

        SQSConnectionFactory connectionFactory = new SQSConnectionFactory(new ProviderConfiguration(), amazonSQS);
        try (SQSConnection connection = connectionFactory.createConnection()) {
            connection.start();

            try (Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE)) {
                String first = "first-queue";
                sendToQueue(session, first, "{\"first-field\": \"first-value\"}");

                String second = "second-queue";
                sendToQueue(session, second, "{\"second-field\": \"second-value\"}");

                trafficParrotAmazonSQS.verifyMessageWasSentToQueueMatchingJsonPath(first, "$.first-field");
                trafficParrotAmazonSQS.verifyMessageWasSentToQueueContainingText(second, "second-value");
            }
        }

        trafficParrotAmazonSQS.purgeQueue("first-queue");
        trafficParrotAmazonSQS.purgeQueue("second-queue");
    }

    private void sendToQueue(Session session, String queueName, String text) throws JMSException {
        Queue queue = session.createQueue(queueName);
        MessageProducer producer = session.createProducer(queue);
        producer.send(session.createTextMessage(text));
        producer.close();
    }
}
